/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the Company) for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

Author          Date    Comments
--------------------------------------------------------------------------------
BC/KO       11-10-2007  Initial creation
KO          03-31-2008  First release

*******************************************************************************/

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"

/**********************************************************************
 * LAB1 Step 3f - USB Include Files                                   *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-comment" the line that includes the "usb.h" header (but not
 *      the one that includes the "usb_host_generic.h" file).
 *  2.  Attempt to re-build the program.  Observe that the errors 
 *      indicate undefined references to "Generic" driver API routines.
 *  3.  "Un-comment" the line that includes the "usb_host_generic.h" 
 *      Generic client driver API header.
 *  4.  Attempt to re-build the program.
 */
// #include "USB/usb.h"
// #include "USB/usb_host_generic.h"
/*
 **********************************************************************
 * END LAB1 Step 3f                                                   *
 **********************************************************************/

#include "user.h"
#include "LCDBlocking.h"
#include "timer.h"


 

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

#if defined(__C30__)

	// Configuration Bit settings
	//      Primary Oscillator:             HS
	//      Internal USB 3.3v Regulator:    Disabled
	//      IOLOCK:                         Set Once	
	//      Primary Oscillator Output:      Digital I/O
	//      Clock Switching and Monitor:    Both disabled
	//      Oscillator:                     Primary with PLL
	//      USB 96MHz PLL Prescale:         Divide by 2
	//      Internal/External Switch Over:  Enabled
	//      WDT Postscaler:                 1:32768
	//      WDT Prescaler:                  1:128
	//      WDT Window:                     Non-window Mode
	//      Comm Channel:                   EMUC2/EMUD2
	//      Clip on Emulation Mode:         Reset into Operation Mode
	//      Write Protect:                  Disabled
	//      Code Protect:                   Disabled
	//      JTAG Port Enable:               Disabled
    
    #define PLL_96MHZ_OFF   0xFFFF
    #define PLL_96MHZ_ON    0xF7FF
    
    
    _CONFIG2(FNOSC_PRIPLL & POSCMOD_HS & PLL_96MHZ_ON & PLLDIV_DIV2) // Primary HS OSC with PLL, USBPLL /2
    _CONFIG1(JTAGEN_OFF & FWDTEN_OFF & ICS_PGx2)   // JTAG off, watchdog timer off

#endif

#if defined(__32MX460F512L__)
        
       	#pragma config UPLLEN   = ON            // USB PLL Enabled
	   	#pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
		#pragma config FPLLMUL  = MUL_15        // PLL Multiplier
	    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
        #pragma config FPLLODIV = DIV_1         // PLL Output Divider
        #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
        #pragma config FWDTEN   = OFF           // Watchdog Timer
        #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
        #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
        #pragma config OSCIOFNC = OFF           // CLKO Enable
        #pragma config POSCMOD  = HS            // Primary Oscillator
        #pragma config IESO     = OFF           // Internal/External Switch-over
        #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
        #pragma config FNOSC    = PRIPLL        // Oscillator Selection
        #pragma config CP       = OFF           // Code Protect
        #pragma config BWP      = OFF           // Boot Flash Write Protect
        #pragma config PWP      = OFF           // Program Flash Write Protect
        #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
        #pragma config DEBUG    = ON            // Background Debugger Enable
        
#endif

// *****************************************************************************
// *****************************************************************************
// Constants
// *****************************************************************************
// *****************************************************************************

#define IOPORT_BIT_15                       (1 << 15)
#define IOPORT_BIT_14                       (1 << 14)
#define IOPORT_BIT_13                       (1 << 13)
#define IOPORT_BIT_12                       (1 << 12)
#define IOPORT_BIT_11                       (1 << 11)
#define IOPORT_BIT_10                       (1 << 10)
#define IOPORT_BIT_9                        (1 << 9)
#define IOPORT_BIT_8                        (1 << 8)
#define IOPORT_BIT_7                        (1 << 7)
#define IOPORT_BIT_6                        (1 << 6)
#define IOPORT_BIT_5                        (1 << 5)
#define IOPORT_BIT_4                        (1 << 4)
#define IOPORT_BIT_3                        (1 << 3)
#define IOPORT_BIT_2                        (1 << 2)
#define IOPORT_BIT_1                        (1 << 1)
#define IOPORT_BIT_0                        (1 << 0)


// *****************************************************************************
// *****************************************************************************
// Data Structures
// *****************************************************************************
// *****************************************************************************

// Switch & LED States
typedef union
{
    BYTE    byte;
    struct _BIT
    {
        BYTE    b0 : 1;
        BYTE    b1 : 1;
        BYTE    b2 : 1;
        BYTE    b3 : 1;
        BYTE    b4 : 1;
        BYTE    b5 : 1;
        BYTE    b6 : 1;
        BYTE    b7 : 1;
    }bits;
}BIT_STATE;


// Application States
typedef enum
{
    DEMO_INITIALIZE = 0,                // Initialize the app when a device is attached
    DEMO_STATE_IDLE,                    // Inactive State


    DEMO_STATE_GET_DEV_VERSION,         // New device attached, send Get-Dev-FW-Version command
    DEMO_STATE_WAITING_VER_REQ,         // Waiting for Get Version command to complete
    DEMO_STATE_READ_DEV_VERSION,        // Start reading Dev FW version data
    DEMO_STATE_WAITING_READ_VER,        // Waiting for read of version data to complete
    DEMO_STATE_VERIFY_DEV_FW_VER,       // Dev FW Ver received, verify & display it

    DEMO_STATE_GET_TEMPERATURE,         // Send get-temperature command
    DEMO_STATE_WAITING_GET_TEMP,        // Waiting for get-temperature command to complete
    DEMO_STATE_READ_TEMPERATURE,        // Start reading temperature data
    DEMO_STATE_WAITING_READ_TEMP,       // Waiting for read of temperature data to complete
    DEMO_STATE_DISPLAY_TEMPERATURE,     // Temperature data received, display it

    DEMO_STATE_GET_POT,                 // Send get-POT command
    DEMO_STATE_WAITING_GET_POT,         // Waiting for get-POT command to complete
    DEMO_STATE_READ_POT,                // Start reading POT data
    DEMO_STATE_WAITING_READ_POT,        // Waiting for read of POT data to complete
    DEMO_STATE_DISPLAY_POT,             // POT data received, display it

    DEMO_STATE_SEND_SET_LED,            // Send set-LED command
    DEMO_STATE_WAITING_SET_LED,         // Waiting for set-LED command to complete
    DEMO_STATE_READ_SET_LED_RESP,       // Start reading set-LED response
    DEMO_STATE_WAITING_LED_RESP,        // Waiting for read of LED response to complete
    DEMO_STATE_UPDATE_LED_STATE,        // Set-LED command successful, update state.

    DEMO_STATE_ERROR                    // An error has occured

} DEMO_STATE;


// *****************************************************************************
// *****************************************************************************
// Macros
// *****************************************************************************
// *****************************************************************************

// Macro to fill a line of LCD data.
#define FILL_LCD_LINE(l,p) {int i; for(i=0;i<16;i++)LcdData[(l)-1][i]=(p)[i];}


// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE        deviceAddress;  // Address of the device on the USB
DATA_PACKET DataPacket;     // Data to send to the device
BYTE        PrevCMD;        // Most recent device command sent.
BYTE        ResponseSize;   // Size of the response expected.
DEMO_STATE  DemoState;      // Current state of the demo application
BYTE        LcdData[2][16]; // LCD Display data
BIT_STATE   LEDState;       // Bitmap of LED states (only 3 & 4 are used)
WORD        SwitchState;    // Bitmap of Switch states (only 3 & 6 are used)

// Initial data for the LCD display.
const char *LcdInitData[] = { "Host -.- Dev -.-", "--.-C ----- Ohms" };


//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{
    #if defined( __PIC24FJ256GB110__ )
        // Configure U2RX - put on pin 49 (RP10)
        RPINR19bits.U2RXR = 10;

        // Configure U2TX - put on pin 50 (RP17)
        RPOR8bits.RP17R = 5;

        OSCCON = 0x3302;    // Enable secondary oscillator
        CLKDIV = 0x0000;    // Set PLL prescaler (1:1)
    #elif defined(__PIC32MX__)
        {
            int  value;
    
            value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
    
            // Enable the cache for the best performance
            CheKseg0CacheOn();
    
            INTEnableSystemMultiVectoredInt();
    
            value = OSCCON;
            while (!(value & 0x00000020))
            {
                value = OSCCON;    // Wait for PLL lock to stabilize
            }
        }
    #endif
    TRISA = 0x0000;
    TRISD = 0x00C0;

    // Init LCD
    LCDInit();

    // Init LEDs
    mInitAllLEDs();
    mLED_9_Off();
    mLED_10_Off();

    // Init Switches
    //mInitAllSwitches();
    SwitchState = IOPORT_BIT_6|IOPORT_BIT_7;

    // Init UART
    UART2Init();

    // Set Default demo state
    DemoState = DEMO_INITIALIZE;

    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    
	/**********************************************************************
    * LAB1 Step 4f - Generic Driver, Detecting Attach                    *
    **********************************************************************
    **************************  B E G I N N E R  *************************
    **********************************************************************
    *  1.  "Un-comment" the lines of code between this comment block and 
    *      the "END" block.
    *  2.  Re-build the application.
    *  3.  Program the application to the microcontroller.
    *  4.  Run the application and observe the change in behaviour.
    *
    
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        DevID.vid   = 0x04D8;
        DevID.pid   = 0x000C;
        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
            return TRUE;
        }
    }

    *
    **********************************************************************
    * END LAB1 Step 4f                                                   *
    **********************************************************************/
    
    return FALSE;

} // CheckForNewAttach


/*************************************************************************
 * Function:        InitLcdMessage
 *
 * Preconditions:   Assumes that INTEnableSystemMultiVectoredInt and
 *                  Ex16LCDInit have been called.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    The LCD display and message strings have been modified
 *
 * Overview:        Displays the App's base LCD message
 *************************************************************************/

void InitLcdMessage ( void )
{
    // Init LCD message
    FILL_LCD_LINE(1, &LcdInitData[0][0]);
    FILL_LCD_LINE(2, &LcdInitData[1][0]);
    LcdData[0][5] = '0' + USB_HOST_FW_MAJOR_VER;
    LcdData[0][7] = '0' + USB_HOST_FW_MINOR_VER;

    // display message
    LCDWriteLine(1, (char *)&LcdData[0][0] );
    LCDWriteLine(2, (char *)&LcdData[1][0] );

} // InitLcdMessage

/*************************************************************************
 Function:      SendDeviceCommand

 Preconditions: Assumes the device is currently attached and enumerated
                with the given address.
                  
 Input:         address USB address of the device.
 
                packet  Pointer to the data packet containing the 
                        command to be sent to the device.
                        
 Output:        None

 Returns:       TRUE if able to send the packet, FALSE if not.

 Side Effects:  If successful, a USB "OUT" transfer has been started.  If
                not successful, the demo has been placed in an error 
                state.  Also, the "PrevCMD" global has been updated with 
                the value of the command given.

 Overview:      This routine sends the given command packet to the device.
*************************************************************************/

BOOL SendDeviceCommand ( BYTE address, DATA_PACKET *packet )
{
    BYTE error;

    /**********************************************************************
     * LAB1 Step 4g - Generic Driver, Transmitting Data                   *
     **********************************************************************
     **************************  B E G I N N E R  *************************
     **********************************************************************
     *  1.  "Un-comment"the lines of code between this and the "END" block
     *      for this lab step.
     *  2.  Re-build the application.
     *  3.  Program the application to the microcontroller.
     *  4.  Run the application and observe the change in behaviour.
     *

    // If the bus isn't busy...
    if (USBHostGenericTxIsComplete(address, &error))
    {
        // Send the command.
        if (USBHostGenericWrite(address, packet, packet->len) == USB_SUCCESS )
        {
            // If successful, save the command for verification.
            PrevCMD     = packet->CMD;
            return TRUE;
        }
        else
        {
            // If not successful, go to an error state.
            LCDWriteLine( 2, "Dev Write Error " );
            DemoState = DEMO_STATE_ERROR;
        }
    }

     *
     **********************************************************************
     * END LAB1 Step 4g                                                   *
     **********************************************************************/

    return FALSE;

} // SendDeviceCommand


/*************************************************************************
 Function:      ReadDeviceResponse

 Preconditions: Assumes the device is currently attached and enumerated
                with the given address.  Also assumes that the device
                has been given a command whose response is of the size
                given in the "length" parameter.
                  
 Input:         address USB address of the device.
 
                packet  Pointer to the data packet to receive the 
                        response.
                        
                length  Number of valid bytes in the response.

 Output:        None

 Returns:       TRUE if able to start the read, FALSE if not.

 Side Effects:  If successful, a USB "IN" transfer has been started.  If
                not successful, the demo has been placed in an error 
                state.

 Overview:      This routine reads the given number of bytes from the
                device.
*************************************************************************/

BOOL ReadDeviceResponse ( BYTE address, DATA_PACKET *packet, BYTE length )
{
    BYTE    errorCode;
    DWORD   byteCount;

    // If the bus isn't busy...
    if (USBHostGenericRxIsComplete(deviceAddress, &errorCode, &byteCount))
    {
        // Initiate the read transfer
        if (USBHostGenericRead(address, packet, length) == USB_SUCCESS )
        {
            ResponseSize = length;
            return TRUE;
        }
        else
        {
            // If not successful, go to an error state.
            LCDWriteLine( 2, "Dev Read Error  " );
            DemoState = DEMO_STATE_ERROR;
        }
    }

    return FALSE;

} // ReadDeviceResponse


/*************************************************************************
 * Function:        ValidateAndDisplayDeviceFwVersion
 *
 * Preconditions:   Assumes the device FW Version response packet has been
 *                  received.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if the Device FW data is valid, FALSE if not.
 *
 * Side Effects:    The LCD display and message strings have been modified.
 *                  Also displays a message on the serial port.
 *
 * Overview:        Validates data in the packet buffer against expected
 *                  device firmware version and displays it on the LCD and
 *                  serial port.
 *************************************************************************/

BOOL ValidateAndDisplayDeviceFwVersion ( void )
{
    if ( USBHostGenericGetRxLength(deviceAddress) == 4             &&
         DataPacket.CMD                   == READ_VERSION          &&
         DataPacket._byte[3]              >= DEMO_FW_MAJOR_VERSION    )
    {
        // Display Device FW version on LCD
        LcdData[0][13] = '0' + DataPacket._byte[3];
        LcdData[0][15] = '0' + DataPacket._byte[4];
        LCDWriteLine(1, (char *)&LcdData[0][0] );

        return TRUE;
    }
    else
    {
        LCDWriteLine( 2, "Dev FW Ver Error" );
        return FALSE;
    }

} // ValidateAndDisplayDeviceFwVersion


/*************************************************************************
 * Function:        DisplayTemperature
 *
 * Preconditions:   Assumes the device FW Version response packet has been
 *                  received.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    The LCD display and message strings have been modified.
 *                  Also displays a message on the serial port.
 *
 * Overview:        Displays the temperature data in the packet buffer on
 *                  the LCD.
 *************************************************************************/

void DisplayTemperature ( void )
{
    DWORD Temperature;
    BYTE   Digit;

    // Calculate the temperature in degrees C
    Temperature  =  DataPacket._byte[1];
    Temperature |= (DataPacket._byte[2] << 8);
    Temperature += 6400;
    Temperature /= 41;

    // Translate to ASCII
    Digit = Temperature % 10;
    Temperature   /= 10;
    LcdData[1][3]  = '0' + Digit;
    Digit = Temperature % 10;
    Temperature   /= 10;
    LcdData[1][1]  = '0' + Digit;
    Digit = Temperature % 10;
    Temperature   /= 10;
    LcdData[1][0]  = '0' + Digit;

    // Display on LCD
    LCDWriteLine(2, (char *)&LcdData[1][0] );

} // DisplayTemperature


/*************************************************************************
 * Function:        DisplayPot
 *
 * Preconditions:   Assumes the device FW Version response packet has been
 *                  received.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    The LCD display and message strings have been modified.
 *                  Also displays a message on the serial port.
 *
 * Overview:        Displays the potentiometer data in the packet buffer on
 *                  the LCD.
 *************************************************************************/

void DisplayPot ( void )
{
    DWORD POT_Val;
    BYTE   Digit;
    int    i;

    // Calculate actual POT value
    POT_Val  =  DataPacket._byte[1];
    POT_Val |= (DataPacket._byte[2] << 8);
    POT_Val *= POT_FULL_VALUE+10;
    POT_Val /= 1024;

    // Translate to ASCII
    for (i=0; i < 5; i++)
    {
        Digit             = POT_Val % 10;
        POT_Val          /= 10;
        LcdData[1][10-i]  = '0' + Digit;
    }

    // Display on LCD
    LCDWriteLine(2, (char *)&LcdData[1][0] );

} // DisplayPot


/*************************************************************************
 Function:        UpdateHostLEDs

 Preconditions:   Assumes the LCD has been initialized and that the data
                  in the packet buffer matches the format described below.
                  
 Input:           packet  Pointer to the data packet containing the 
                          LED data.

 Output:          None

 Returns:         TRUE if the packet is valid, FALSE if not.

 Side Effects:    The LEDs have been updated.

 Overview:        This routine updates the state of the LEDs to match 
                  that given in the packet.
                  
  *************************************************************************/

BOOL UpdateHostLEDs ( DATA_PACKET *packet )
{
    // If the CMD is valid...
    if (packet->CMD == UPDATE_LED)
    {
        // led_num left-over from set-LED command send
        if (packet->led_num == 3)
        {
            LEDState.bits.b3 ^= 1;
            mLED_10_Toggle();
        }
        else if (packet->led_num == 4)
        {
            LEDState.bits.b4 ^= 1;
            mLED_9_Toggle();
        }
        return TRUE;
    }

    // Otherwise, report error.
    LCDWriteLine( 2, "Invalid LED Resp" );
    DemoState = DEMO_STATE_ERROR;
    return FALSE;

} // UpdateHostLEDs

/*************************************************************************
 * Function:        Switch3WasPressed & Switch6WasPressed
 *
 * Preconditions:   Assumes SwitchState & Switches have been initialized
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if the switch has been pressed, FALSE otherwise
 *
 * Side Effects:    SwitchState has been updated
 *
 * Overview:        Determines if the switched named has been pressed
 *                  since last time the routine was called.
 *************************************************************************/

BOOL Switch3WasPressed(void)
{
    WORD sw3_state;

    sw3_state = PORTD & IOPORT_BIT_6;

    if( sw3_state != (SwitchState & IOPORT_BIT_6) )
    {
        SwitchState &= ~IOPORT_BIT_6;
        SwitchState |= sw3_state;

        if(SwitchState & IOPORT_BIT_6)
            return TRUE;                // Was pressed
    }

    return FALSE;                       // Was not pressed

} // Switch3WasPressed

BOOL Switch6WasPressed(void)
{
    WORD sw6_state;

    sw6_state = PORTD & IOPORT_BIT_7;

    if( sw6_state != (SwitchState & IOPORT_BIT_7) )
    {
        SwitchState &= ~IOPORT_BIT_7;
        SwitchState |= sw6_state;

        if(SwitchState & IOPORT_BIT_7)
            return TRUE;                // Was pressed
    }

    return FALSE;                       // Was not pressed

}//end Switch6WasPressed

/*************************************************************************
 Function:        UpdateDeviceLEDs

 Preconditions:   Assumes that the 
                  
 Input:           None

 Output:          None

 Returns:         TRUE if a command was sent to the device to change its
                  LED state, FALSE if not.

 Side Effects:    If either SW3 or SW6 was pressed, the data buffer will 
                  have been updated to hold the command sent to the device
                  to update the device's LEDs.

 Overview:        This routine checks to see if SW3 or SW6 was pressed and
                  (if either was pressed) sends the appropriate command 
                  to the device to toggle the state of it's LEDs
                  
                    * SW3 on Host toggles D3 on Device
                    * SW6 on Host toggles D4 on Device
                  
 Note:            This routine is only called once from "ManageDemoState",
                  so it is defined as a "static inline" routine to avoid 
                  function call overhead.
                  
                  If an "UPDATE_LED" command was sent to the device, the 
                  caller must read the expected response.
                  
                  Only a single "UPDATE_LED" command can be sent to the 
                  device at a time, so this routine will only identify a 
                  single switch press at a time (giving priority to SW3).
 *************************************************************************/

BOOL UpdateDeviceLEDs ( void )
{
    // Send the set-LED command
    DataPacket.CMD = UPDATE_LED;
    DataPacket.len = UPDATE_LED_CMD_SIZE;
    if (Switch3WasPressed())
    {
        DataPacket.led_num    = 3;      // LED 3 on PIC18 FS USB board
        DataPacket.led_status = LEDState.bits.b3 ^ 1;
    }
    else if (Switch6WasPressed())
    {
        DataPacket.led_num    = 4;      // LED 4 on PIC18 FS USB board
        DataPacket.led_status = LEDState.bits.b4 ^ 1;
    }
    else
    {
        return FALSE;
    }

    if (SendDeviceCommand(deviceAddress, &DataPacket))
    {
        return TRUE;
    }

    return FALSE;

} // UpdateDeviceLEDs


/******************************************************************************
 * Function:        void BlinkStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkStatus turns on and off LEDs corresponding to
 *                  the demo state.
 *
 * Note:
 *****************************************************************************/
void BlinkStatus(void)
{
    static unsigned int led_count=0;

    if(led_count == 0)
        led_count = 3000;

    led_count--;

    if(DemoState <= DEMO_STATE_IDLE)
    {
        mLED_5_Off();
        mLED_6_Off();
    }
    else if(DemoState <= DEMO_STATE_VERIFY_DEV_FW_VER)
    {
        mLED_5_On();
        mLED_6_On();
    }
    else if(DemoState < DEMO_STATE_ERROR)
    {
        if (led_count == 0)
        {
            mLED_5_Toggle();
            mLED_6 = !mLED_5;       // Alternate blink
        }
    }
    else    // DEMO_STATE_ERROR
    {
        if (led_count == 0)
        {
            mLED_5_Toggle();
            mLED_6 = mLED_5;        // Both blink at the same time
            led_count = 50;
        }

        DelayMs(1); // 1ms delay
    }

}//end BlinkStatus


/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *
 * Note:            This routine is only called from "main", so it is 
 *                  defined as a "static inline" routine to avoid function
 *                  call overhead.
 *************************************************************************/
static inline void ManageDemoState ( void )
{
    BYTE        errorCode;
	DWORD       byteCount;
	
    BlinkStatus();

    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
        DemoState = DEMO_INITIALIZE;
        deviceAddress   = 0;
    }

    switch (DemoState)
    {
    case DEMO_INITIALIZE:
        InitLcdMessage();
        DemoState = DEMO_STATE_IDLE;
        break;

    /** Idle State:  Loops here until attach **/
    case DEMO_STATE_IDLE:
        if (CheckForNewAttach())
        {
            DemoState = DEMO_STATE_GET_DEV_VERSION;
        }
        break;

    /** Sequence: Read Dev FW Version **/
    case DEMO_STATE_GET_DEV_VERSION:
        // Send the Read Version command
        DataPacket.CMD = READ_VERSION;
        DataPacket.len = READ_VERSION_CMD_SIZE;
        if (SendDeviceCommand(deviceAddress, &DataPacket))
            DemoState = DEMO_STATE_WAITING_VER_REQ;
        break;

    case DEMO_STATE_WAITING_VER_REQ:
        if (USBHostGenericTxIsComplete(deviceAddress, &errorCode) )
            DemoState = DEMO_STATE_READ_DEV_VERSION;
        break;

    case DEMO_STATE_READ_DEV_VERSION:
        if (ReadDeviceResponse(deviceAddress, &DataPacket, READ_VERSION_RESP_SIZE))
            DemoState = DEMO_STATE_WAITING_READ_VER;
        break;

    case DEMO_STATE_WAITING_READ_VER:
        if (USBHostGenericRxIsComplete(deviceAddress, &errorCode, &byteCount))
            DemoState = DEMO_STATE_VERIFY_DEV_FW_VER;
        break;

    case DEMO_STATE_VERIFY_DEV_FW_VER:
        if (ValidateAndDisplayDeviceFwVersion())
            DemoState = DEMO_STATE_GET_TEMPERATURE;
        else
            DemoState = DEMO_STATE_ERROR;
        break;

    /** Sequence: Read Temperature Sensor Data **/
    case DEMO_STATE_GET_TEMPERATURE:
        // Send the Read Temperature command
        DataPacket.CMD = RD_TEMP;
        DataPacket.len = RD_TEMP_CMD_SIZE;
        if (SendDeviceCommand(deviceAddress, &DataPacket))
            DemoState = DEMO_STATE_WAITING_GET_TEMP;
        break;

    case DEMO_STATE_WAITING_GET_TEMP:
        if (USBHostGenericTxIsComplete(deviceAddress, &errorCode) )
            DemoState = DEMO_STATE_READ_TEMPERATURE;
        break;

    case DEMO_STATE_READ_TEMPERATURE:
        if (ReadDeviceResponse(deviceAddress, &DataPacket, RD_TEMP_RESP_SIZE))
            DemoState = DEMO_STATE_WAITING_READ_TEMP;
        break;
        
    case DEMO_STATE_WAITING_READ_TEMP:
        if (USBHostGenericRxIsComplete(deviceAddress, &errorCode, &byteCount))
            DemoState = DEMO_STATE_DISPLAY_TEMPERATURE;
        break;

    case DEMO_STATE_DISPLAY_TEMPERATURE:
        DisplayTemperature();
        DemoState = DEMO_STATE_GET_POT;
        break;

    /** Sequence: Read POT Sensor Data **/
    
    /**********************************************************************
     * LAB1 Step 5a - To Do:  Add Missing "Read Pot" Feature              *
     **********************************************************************
     **********************  I N T E R M E D I A T E  *********************
     **********************************************************************
     *  1.  Remove the temporary implementation of the DEMO_STATE_GET_POT
     *      state transition, directly below this comment header.
     *  2.  Add the "Sequence to read POT Sensor Data" (below) into the 
     *      program in place of the original temporary implementation.
     *  3.  Re-build the application.
     *  4.  Program the application to the microcontroller.
     *  5.  Run the application and observe the change in behaviour.
     *  6.  Stop the running application and set 5 breakpoints, one on each
     *      of the four new state transitions plus one on the
     *      DEMO_STATE_DISPLAY_POT state transition.
     *  7.  Reset the microcontroller and run the application.
     *  8.  When a breakpoint is hit, step through the code as desired.
     *      Continue running once you have stepped into the Generic client
     *      driver.  Do not step into the usb_host.c at this time.
     *  9.  Observe the sequence of "tasks" that are necessary to read and
     *      display the potentiometer value.
     */

    case DEMO_STATE_GET_POT:
        DataPacket.CMD      = RD_POT;
        DataPacket._byte[2] = 0;
        DataPacket._byte[1] = 0;
        DemoState = DEMO_STATE_DISPLAY_POT;
        break;

    /*
     * Sequence to read POT Sensor Data
     *
     
    case DEMO_STATE_GET_POT:
        // Send the Read POT command
        DataPacket.CMD = RD_POT;
        DataPacket.len = RD_POT_CMD_SIZE;
        if (SendDeviceCommand(deviceAddress, &DataPacket))
            DemoState = DEMO_STATE_WAITING_GET_POT;
        break;

    case DEMO_STATE_WAITING_GET_POT:
        if (USBHostGenericTxIsComplete(deviceAddress, &errorCode) )
            DemoState = DEMO_STATE_READ_POT;
        break;

    case DEMO_STATE_READ_POT:
        if (ReadDeviceResponse(deviceAddress, &DataPacket, RD_POT_RESP_SIZE))
            DemoState = DEMO_STATE_WAITING_READ_POT;
        break;

    case DEMO_STATE_WAITING_READ_POT:
        if (USBHostGenericRxIsComplete(deviceAddress, &errorCode, &byteCount))
            DemoState = DEMO_STATE_DISPLAY_POT;
        break;
        
     *
     *  END Sequence to read POT Sensor Data
     */

    /**********************************************************************
     * END LAB1 Step 5a                                                   *
     **********************************************************************/

    case DEMO_STATE_DISPLAY_POT:
        DisplayPot();
        DemoState = DEMO_STATE_GET_TEMPERATURE;
        break;

    /** Sequence:  Update LEDs **/
    
    /**********************************************************************
     * LAB1 Step 6a - Add New Feature                                     *
     **********************************************************************
     **************************  A D V A N C E D  *************************
     **********************************************************************
     * Add new code as needed to monitor SW3 and SW6 and send commands to 
     * the device to toggle LEDs D3 and D4 respectively when these switches
     * have been pressed.  Once you know the command has been received by 
     * the device (by the fact that you've read the response from the 
     * device), set the host's LEDs D10 and D9 respectively to demonstrate 
     * that the host and device are in sync.
     *
     * Hints:   The "UpdateDeviceLEDs()" routine has been implemented for 
     *          you to monitor the switches and send the command to the
     *          device.
     *
     *          The "UpdateHostLEDs()" routine has been implemented for you
     *          to keep the host LEDs in sync with the device's LEDs.
     * 
     * NOTE: 	You must enable the main application state machine to enter
     *			this section by setting the DemoState transition state to 
     *			"DEMO_STATE_SEND_SET_LED" in the "DEMO_STATE_DISPLAY_POT" 
     *			action above. 
     *  
     */

    // <--- Add sequence to update the LEDs here.

    /**********************************************************************
     * END LAB1 Step 6a                                                   *
     **********************************************************************/

    /** Error state:  Hold here until detached **/
    case DEMO_STATE_ERROR:                          // To Do: Flash LEDs
        break;

    default:
        DemoState = DEMO_INITIALIZE;
        break;
    }

    DelayMs(1); // 1ms delay

} // ManageDemoState

//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    
    /**********************************************************************
     * LAB1 Step 4e - USB Initilziation and Tasks                         *
     **********************************************************************
     **************************  B E G I N N E R  *************************
     **********************************************************************
     *  1.  "Un-comment calls to "USBInitialize" and "USBTasks" (below).
     *  2.  Re-build the application.
     *  3.  Program the application to the microcontroller.
     *  4.  Run the application and observe the change in behaviour.
     */

    // Initialize the USB
    // USBInitialize(0);
    
    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        // USBTasks();
        
        // Maintain Demo Application State
        ManageDemoState();
    }

	/**********************************************************************
     * END LAB1 Step 4e                                                   *
     **********************************************************************/

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

