/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the Company) for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

Author          Date    Comments
--------------------------------------------------------------------------------
BC/KO       11-10-2007  Initial creation
KO          03-31-2008  First release

*******************************************************************************/

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "USB/usb.h"
#include "usb_client_generic.h"
#include "user.h"
#include "LCDBlocking.h"
#include "timer.h"

/*************************
 * Appliation Parameters *
 *************************/

// Max Current Allowed for VBus
#define MAX_ALLOWED_CURRENT     100     // mA

// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

#if defined(__C30__)

	// Configuration Bit settings
	//      Primary Oscillator:             HS
	//      Internal USB 3.3v Regulator:    Disabled
	//      IOLOCK:                         Set Once	
	//      Primary Oscillator Output:      Digital I/O
	//      Clock Switching and Monitor:    Both disabled
	//      Oscillator:                     Primary with PLL
	//      USB 96MHz PLL Prescale:         Divide by 2
	//      Internal/External Switch Over:  Enabled
	//      WDT Postscaler:                 1:32768
	//      WDT Prescaler:                  1:128
	//      WDT Window:                     Non-window Mode
	//      Comm Channel:                   EMUC2/EMUD2
	//      Clip on Emulation Mode:         Reset into Operation Mode
	//      Write Protect:                  Disabled
	//      Code Protect:                   Disabled
	//      JTAG Port Enable:               Disabled
    
    #define PLL_96MHZ_OFF   0xFFFF
    #define PLL_96MHZ_ON    0xF7FF
    
    
    _CONFIG2(FNOSC_PRIPLL & POSCMOD_HS & PLL_96MHZ_ON & PLLDIV_DIV2) // Primary HS OSC with PLL, USBPLL /2
    _CONFIG1(JTAGEN_OFF & FWDTEN_OFF & ICS_PGx2)   // JTAG off, watchdog timer off

#endif

#if defined(__32MX460F512L__)
        
        #pragma config UPLLEN   = ON            // USB PLL Enabled
	    #pragma config UPLLIDIV = DIV_2         // USB PLL Input Divider
        #pragma config FPLLMUL  = MUL_15        // PLL Multiplier
	    #pragma config FPLLIDIV = DIV_2         // PLL Input Divider
        #pragma config FPLLODIV = DIV_1         // PLL Output Divider
        #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
        #pragma config FWDTEN   = OFF           // Watchdog Timer
        #pragma config WDTPS    = PS1           // Watchdog Timer Postscale
        #pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
        #pragma config OSCIOFNC = OFF           // CLKO Enable
        #pragma config POSCMOD  = HS            // Primary Oscillator
        #pragma config IESO     = OFF           // Internal/External Switch-over
        #pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable (KLO was off)
        #pragma config FNOSC    = PRIPLL        // Oscillator Selection
        #pragma config CP       = OFF           // Code Protect
        #pragma config BWP      = OFF           // Boot Flash Write Protect
        #pragma config PWP      = OFF           // Program Flash Write Protect
        #pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
        #pragma config DEBUG    = ON            // Background Debugger Enable
        
#endif

// *****************************************************************************
// *****************************************************************************
// Constants
// *****************************************************************************
// *****************************************************************************

#define IOPORT_BIT_15                       (1 << 15)
#define IOPORT_BIT_14                       (1 << 14)
#define IOPORT_BIT_13                       (1 << 13)
#define IOPORT_BIT_12                       (1 << 12)
#define IOPORT_BIT_11                       (1 << 11)
#define IOPORT_BIT_10                       (1 << 10)
#define IOPORT_BIT_9                        (1 << 9)
#define IOPORT_BIT_8                        (1 << 8)
#define IOPORT_BIT_7                        (1 << 7)
#define IOPORT_BIT_6                        (1 << 6)
#define IOPORT_BIT_5                        (1 << 5)
#define IOPORT_BIT_4                        (1 << 4)
#define IOPORT_BIT_3                        (1 << 3)
#define IOPORT_BIT_2                        (1 << 2)
#define IOPORT_BIT_1                        (1 << 1)
#define IOPORT_BIT_0                        (1 << 0)


// *****************************************************************************
// *****************************************************************************
// Data Structures
// *****************************************************************************
// *****************************************************************************

// Switch & LED States
typedef union
{
    BYTE    byte;
    struct _BIT
    {
        BYTE    b0 : 1;
        BYTE    b1 : 1;
        BYTE    b2 : 1;
        BYTE    b3 : 1;
        BYTE    b4 : 1;
        BYTE    b5 : 1;
        BYTE    b6 : 1;
        BYTE    b7 : 1;
    }bits;
}BIT_STATE;


// Application States
typedef enum 
{
    DEMO_INITIALIZE = 0,                // Initialize the app when a device is attached
    DEMO_STATE_IDLE,                    // Inactive State
    DEMO_STATE_RUNNING,                 // Running normally between this value and DEMO_STATE_ERROR

    DEMO_STATE_GET_DEV_VERSION,         // New device attached, get device FW version

    DEMO_STATE_GET_TEMPERATURE,         // Get device temperature

    DEMO_STATE_GET_POT,                 // Get potentiometer value

    DEMO_STATE_SET_LED,                 // Set device LEDs

    DEMO_STATE_ERROR                    // An error has occured

} DEMO_STATE;


// *****************************************************************************
// *****************************************************************************
// Macros
// *****************************************************************************
// *****************************************************************************

// Macro to fill a line of LCD data.
#define FILL_LCD_LINE(l,p) {int i; for(i=0;i<16;i++)LcdData[(l)-1][i]=(p)[i];}

/*********************************
 * USB Host: Client Driver Table *
 *********************************/

CLIENT_DRIVER_TABLE usbClientDrvTable[] = 
{
    {   // Generic Client Driver
        USBHostGenericInit,           // Initialization Routine
        USBHostGenericEventHandler,   // Event Handler Routine
        0                             // Initializaton Parameter
    }
};

/************************************
 * USB Host: Target Peripheral List *
 ************************************/

USB_TPL usbTPL[NUM_TPL_ENTRIES] = 
{
//       VID & PID or                           Client
//       Class, Subclass & Protocol      Config Driver  Flags
    { INIT_VID_PID( 0x04D8ul, 0x000Cul ),   0,     0,    {0} }  // Microchip Custom demo
};


// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE        DevAddr;  	// Address of the device on the USB
DATA_PACKET DataPacket;     // Data to send to the device
BYTE        ResponseSize;   // Size of the response expected.
DEMO_STATE  DemoState;      // Current state of the demo application
BYTE        LcdData[2][16]; // LCD Display data
BIT_STATE   LEDState;       // Bitmap of LED states (only 3 & 4 are used)
WORD        SwitchState;    // Bitmap of Switch states (only 3 & 6 are used)

// Initial data for the LCD display.
const char *LcdInitData[] = { "Host -.- Dev -.-", "--.-C ----- Ohms" };


//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{
    #if defined( __PIC24FJ256GB110__ )
        // Configure U2RX - put on pin 49 (RP10)
        RPINR19bits.U2RXR = 10;

        // Configure U2TX - put on pin 50 (RP17)
        RPOR8bits.RP17R = 5;

        OSCCON = 0x3302;    // Enable secondary oscillator
        CLKDIV = 0x0000;    // Set PLL prescaler (1:1)
    #elif defined(__PIC32MX__)
        {
            int  value;
    
            value = SYSTEMConfigWaitStatesAndPB( GetSystemClock() );
    
            // Enable the cache for the best performance
            CheKseg0CacheOn();
    
            INTEnableSystemMultiVectoredInt();
    
            value = OSCCON;
            while (!(value & 0x00000020))
            {
                value = OSCCON;    // Wait for PLL lock to stabilize
            }
        }
    #endif
    TRISA = 0x0000;
    TRISD = 0x00C0;

    // Init LCD
    LCDInit();

    // Init LEDs
    mInitAllLEDs();
    mLED_9_Off();
    mLED_10_Off();

    // Init Switches
    //mInitAllSwitches();
    SwitchState = IOPORT_BIT_6|IOPORT_BIT_7;

    // Init UART
    UART2Init();

    // Set Default demo state
    DemoState = DEMO_INITIALIZE;

    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        InitLcdMessage
 *
 * Preconditions:   Assumes that INTEnableSystemMultiVectoredInt and
 *                  Ex16LCDInit have been called.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         None
 *
 * Side Effects:    The LCD display and message strings have been modified
 *
 * Overview:        Displays the App's base LCD message
 *************************************************************************/

void InitLcdMessage ( void )
{
    // Init LCD message
    FILL_LCD_LINE(1, &LcdInitData[0][0]);
    FILL_LCD_LINE(2, &LcdInitData[1][0]);
    LcdData[0][5] = '0' + USB_HOST_FW_MAJOR_VER;
    LcdData[0][7] = '0' + USB_HOST_FW_MINOR_VER;

    // display message
    LCDWriteLine(1, (char *)&LcdData[0][0] );
    LCDWriteLine(2, (char *)&LcdData[1][0] );

} // InitLcdMessage

/*************************************************************************
 Function:      SendDeviceCommand

 Preconditions: Assumes the device is currently attached and enumerated
                with the given address.
                  
 Input:         address USB address of the device.
 
                packet  Pointer to the data packet containing the 
                        command to be sent to the device.
                        
 Output:        None

 Returns:       TRUE if able to send the packet, FALSE if not.

 Side Effects:  If successful, a USB "OUT" transfer has been started.  If
                not successful, the demo has been placed in an error 
                state.  Also, the "PrevCMD" global has been updated with 
                the value of the command given.

 Overview:      This routine sends the given command packet to the device.
*************************************************************************/

BOOL SendDeviceCommand ( BYTE address, DATA_PACKET *packet )
{
    // Send the command.
    if (USBHostGenericWrite(address, packet, packet->len) == USB_SUCCESS )
    {
        return TRUE;
    }

    // If not successful, go to an error state.
    LCDWriteLine( 2, "Dev Write Error " );
    DemoState = DEMO_STATE_ERROR;

    return FALSE;

} // SendDeviceCommand


/*************************************************************************
 Function:      ReadDeviceResponse

 Preconditions: Assumes the device is currently attached and enumerated
                with the given address.  Also assumes that the device
                has been given a command whose response is of the size
                given in the "length" parameter.
                  
 Input:         address USB address of the device.
 
                packet  Pointer to the data packet to receive the 
                        response.
                        
                length  Number of valid bytes in the response.

 Output:        None

 Returns:       TRUE if able to start the read, FALSE if not.

 Side Effects:  If successful, a USB "IN" transfer has been started.  If
                not successful, the demo has been placed in an error 
                state.

 Overview:      This routine reads the given number of bytes from the
                device.
*************************************************************************/

BOOL ReadDeviceResponse ( BYTE address, DATA_PACKET *packet, BYTE length )
{
     // Initiate the read transfer
    if (USBHostGenericRead(address, packet, length) == USB_SUCCESS )
    {
        ResponseSize = length;
        return TRUE;
    }

    // If not successful, go to an error state.
    LCDWriteLine( 2, "Dev Read Error  " );
    DemoState = DEMO_STATE_ERROR;

    return FALSE;

} // ReadDeviceResponse


/*************************************************************************
 * Function:        ValidateAndDisplayDeviceFwVersion
 *
 * Preconditions:   Assumes the device FW Version response packet has been
 *                  received. Refer to COM3202 lab manual appendix for details
 *					of the "FW Version" command/response packet protocol
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if the Device FW data is valid, FALSE if not.
 *
 * Side Effects:    The LCD display and message strings have been modified.
 *                  Also displays a message on the serial port.
 *
 * Overview:        Validates data in the packet buffer against expected
 *                  device firmware version and displays it on the LCD and
 *                  serial port.
 *************************************************************************/

BOOL ValidateAndDisplayDeviceFwVersion ( DATA_PACKET *packet )
{
    // Display Device FW version on LCD (whether it's correct or not)
    LcdData[0][13] = '0' + packet->_byte[3];
    LcdData[0][15] = '0' + packet->_byte[4];
    LCDWriteLine(1, &LcdData[0][0] );

    // Was the data valid and was the version number supported?
    if ( packet->CMD      == READ_VERSION          &&
         packet->len      == 2                     &&
         packet->_byte[3] >= DEMO_FW_MAJOR_VERSION   )
    {
        return TRUE;
    }

    // Otherwise, indicate error
    LCDWriteLine( 2, "Dev FW Ver Error" );
    DemoState = DEMO_STATE_ERROR;
    return FALSE;

} // ValidateAndDisplayDeviceFwVersion

/*************************************************************************
 Function:        DisplayTemperature

 Preconditions:   Assumes the LCD has been initialized and that the data
                  matches the format described below.
                  
 Input:           packet  Pointer to the data packet containing the 
                          temperature data.

 Output:          None

 Returns:         TRUE if the response was valid, FALSE if not.

 Side Effects:    The LCD display and message strings have been modified.

 Overview:        Displays the temperature data in the packet buffer on 
                  the LCD.

 Notes:           Temperature is a 16 bit number representing degrees C.
                  Bits 0 & 1 are not valid.  Bit 2 indicates temperature
                  is valid.  The remaining 13 bits represent degrees 
                  Celsius, in 1/16th (0.0625) degree increments.  In a 
                  fixed point coding scheme, the Binary-point is between
                  bits 6 and 7
                  
                  This routine is only called once from "ManageDemoState",
                  so it is defined as a "static inline" routine to avoid 
                  function call overhead.
 *************************************************************************/

BOOL DisplayTemperature ( DATA_PACKET *packet )
{
    UINT32 Temperature;
    BYTE   Digit;

    // Validate the response packet.
    if ( packet->CMD != RD_TEMP )
    {
        LCDWriteLine( 2, "Bad Temp Packet " );
        DemoState = DEMO_STATE_ERROR;
        return FALSE;
    }

    // If the data is valid, display the temp in degrees C.
    if ( packet->_byte[1] & TEMP_VALID_FLAG  )
    {
        // Calculate the fractional digit.
        Temperature = 500;
        if ( DataPacket._byte[1] & 0x40 ) Temperature += 5000;
        if ( DataPacket._byte[1] & 0x20 ) Temperature += 2500;
        if ( DataPacket._byte[1] & 0x10 ) Temperature += 1250;
        if ( DataPacket._byte[1] & 0x80 ) Temperature += 0625;
        Digit = Temperature / 1000;
        LcdData[1][3]  = '0' + Digit;

        // Formulate the non-fractional portion of the temperature
        Temperature   = DataPacket._byte[2];
        Temperature <<= 1;
        if (DataPacket._byte[1] & 0x80)
            Temperature |= 1;

        // Translate to decimal in ASCII
        Digit = Temperature % 10;
        Temperature   /= 10;
        LcdData[1][1]  = '0' + Digit;
        Digit = Temperature % 10;
        Temperature   /= 10;
        LcdData[1][0]  = '0' + Digit;

        // If we're out of range, blank out the data.
        if (Temperature)
        {
            LcdData[1][3]  = '-';
            LcdData[1][1]  = '-';
            LcdData[1][0]  = '-';
        }

        // Display on LCD
        LCDWriteLine(2, &LcdData[1][0] );
    }

    return TRUE;

} // DisplayTemperature


/*************************************************************************
 Function:        DisplayPot
 
 Preconditions:   Assumes the device FW Version response packet has been
                  received.
                  
 Input:           packet  Pointer to the data packet containing the 
                          potentiometer data.
 
 Output:          None
 
 Returns:         TRUE if the packet is valid, FALSE if not.
 
 Side Effects:    The LCD display and message strings have been modified.
 
 Overview:        Displays the potentiometer data in the packet buffer on 
                  the LCD.
                  
 Note:            This routine is only called once from "ManageDemoState",
                  so it is defined as a "static inline" routine to avoid 
                  function call overhead.
 *************************************************************************/

BOOL DisplayPot ( DATA_PACKET *packet )
{
    UINT32 POT_Val;
    BYTE   Digit;
    int    i;

    // If the data is valid...
    if ( packet->CMD == RD_POT  )
    {
        // Calculate actual POT value
        POT_Val   = packet->_byte[2];
        POT_Val <<= 8;
        POT_Val  |=  packet->_byte[1];
        POT_Val  *= POT_FULL_VALUE+10;
        POT_Val  /= 1024;

        // Translate to ASCII
        for (i=0; i < 5; i++)
        {
            Digit             = POT_Val % 10;
            POT_Val          /= 10;
            LcdData[1][10-i]  = '0' + Digit;
        }

        // Display on LCD
        LCDWriteLine(2, &LcdData[1][0] );
        return TRUE;
    }

    // Otherwise, report error.
    LCDWriteLine( 2, "Invalid POT Resp" );
    DemoState = DEMO_STATE_ERROR;
    return FALSE;

} // DisplayPot



/*************************************************************************
 Function:        UpdateHostLEDs

 Preconditions:   Assumes the LCD has been initialized and that the data
                  in the packet buffer matches the format described below.
                  
 Input:           packet  Pointer to the data packet containing the 
                          LED data.

 Output:          None

 Returns:         TRUE if the packet is valid, FALSE if not.

 Side Effects:    The LEDs have been updated.

 Overview:        This routine updates the state of the LEDs to match 
                  that given in the packet.
                  
  *************************************************************************/

BOOL UpdateHostLEDs ( DATA_PACKET *packet )
{
    // If the CMD is valid...
    if (packet->CMD == UPDATE_LED)
    {
        // led_num left-over from set-LED command send
        if (packet->led_num == 3)
        {
            LEDState.bits.b3 ^= 1;
            mLED_10_Toggle();
        }
        else if (packet->led_num == 4)
        {
            LEDState.bits.b4 ^= 1;
            mLED_9_Toggle();
        }
        return TRUE;
    }

    // Otherwise, report error.
    LCDWriteLine( 2, "Invalid LED Resp" );
    DemoState = DEMO_STATE_ERROR;
    return FALSE;

} // UpdateHostLEDs

/*************************************************************************
 * Function:        Switch3WasPressed & Switch6WasPressed
 *
 * Preconditions:   Assumes SwitchState & Switches have been initialized
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if the switch has been pressed, FALSE otherwise
 *
 * Side Effects:    SwitchState has been updated
 *
 * Overview:        Determines if the switched named has been pressed
 *                  since last time the routine was called.
 *************************************************************************/

BOOL Switch3WasPressed(void)
{
    WORD sw3_state;

    sw3_state = PORTD & IOPORT_BIT_6;

    if( sw3_state != (SwitchState & IOPORT_BIT_6) )
    {
        SwitchState &= ~IOPORT_BIT_6;
        SwitchState |= sw3_state;

        if(SwitchState & IOPORT_BIT_6)
            return TRUE;                // Was pressed
    }

    return FALSE;                       // Was not pressed

} // Switch3WasPressed

BOOL Switch6WasPressed(void)
{
    WORD sw6_state;

    sw6_state = PORTD & IOPORT_BIT_7;

    if( sw6_state != (SwitchState & IOPORT_BIT_7) )
    {
        SwitchState &= ~IOPORT_BIT_7;
        SwitchState |= sw6_state;

        if(SwitchState & IOPORT_BIT_7)
            return TRUE;                // Was pressed
    }

    return FALSE;                       // Was not pressed

}//end Switch6WasPressed

/*************************************************************************
 Function:        UpdateDeviceLEDs

 Preconditions:   Assumes that the 
                  
 Input:           None

 Output:          None

 Returns:         TRUE if a command was sent to the device to change its
                  LED state, FALSE if not.

 Side Effects:    If either SW3 or SW6 was pressed, the data buffer will 
                  have been updated to hold the command sent to the device
                  to update the device's LEDs.

 Overview:        This routine checks to see if SW3 or SW6 was pressed and
                  (if either was pressed) sends the appropriate command 
                  to the device to toggle the state of it's LEDs
                  
                    * SW3 on Host toggles D3 on Device
                    * SW6 on Host toggles D4 on Device
                  
 Note:            This routine is only called once from "ManageDemoState",
                  so it is defined as a "static inline" routine to avoid 
                  function call overhead.
                  
                  If an "UPDATE_LED" command was sent to the device, the 
                  caller must read the expected response.
                  
                  Only a single "UPDATE_LED" command can be sent to the 
                  device at a time, so this routine will only identify a 
                  single switch press at a time (giving priority to SW3).
 *************************************************************************/

BOOL UpdateDeviceLEDs ( void )
{
    // Send the set-LED command
    DataPacket.CMD = UPDATE_LED;
    DataPacket.len = UPDATE_LED_CMD_SIZE;
    if (Switch3WasPressed())
    {
        DataPacket.led_num    = 3;      // LED 3 on PIC18 FS USB board
        DataPacket.led_status = LEDState.bits.b3 ^ 1;
    }
    else if (Switch6WasPressed())
    {
        DataPacket.led_num    = 4;      // LED 4 on PIC18 FS USB board
        DataPacket.led_status = LEDState.bits.b4 ^ 1;
    }
    else
    {
        return FALSE;
    }

    if (SendDeviceCommand(DevAddr, &DataPacket))
    {
        return TRUE;
    }

    return FALSE;

} // UpdateDeviceLEDs


/******************************************************************************
 * Function:        void BlinkStatus(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        BlinkStatus turns on and off LEDs corresponding to
 *                  the demo state.
 *
 * Note:
 *****************************************************************************/
void BlinkStatus(void)
{
    static unsigned int led_count=0;

    if(led_count == 0)
        led_count = 30000;

    led_count--;

    if(DemoState <= DEMO_STATE_IDLE)
    {
        mLED_5_Off();
        mLED_6_Off();
    }
    else if(DemoState >= DEMO_STATE_RUNNING)
    {
        if(DemoState < DEMO_STATE_ERROR)
        {
            if (led_count == 0 )
            {
                mLED_5_Toggle();
                mLED_6 = !mLED_5;       // Alternate blink                
            }
        }
        else    // DEMO_STATE_ERROR
        {
            if (led_count == 0 )
            {
                mLED_5_Toggle();
                mLED_6 = mLED_5;        // Both blink at the same time
                led_count = 50;
            }
            DelayMs(1); // 1ms delay
        }
    }

}//end BlinkStatus

/**************************
 * Main Application Logic *
 **************************/

/*************************************************************************
 Function:        ManageDemoState
 
 Preconditions:   The DemoState global variable must be initialized to
                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
                  
 Input:           DemoState (global)
                  Actions selected based value of DemoState on function
                  entry.
 
                  DevAddr (global)
                  May use device address to access device, depending on
                  state.
 
                  DataPacket (global)
                  May read data from packet buffer, depending on state.
 
 Output:          DemoState (global)
                  Updates demo state as appropriate.
 
                  DataPacket (global)
                  May cause data in the packet buffer to be updated, 
                  depending on state.
 
 Returns:         None
 
 Side Effects:    Depend on state transition
 
 Overview:        This routine maintains the state of the application,
                  updateing global data and taking actions as necessary
                  to maintain the custom demo operations.
                  
 Note:            This routine is only called from "main", so it is 
                  defined as a "static inline" routine to avoid function
                  call overhead.
 *************************************************************************/
static inline void ManageDemoState ( void )
{
    BlinkStatus();

    // Main Application State Machine
    switch (DemoState)
    {
    // ** Initial state of the demo **
    case DEMO_INITIALIZE:                   // Starts here after reset or detach
        InitLcdMessage();
        DemoState = DEMO_STATE_IDLE;
        break;

    // ** Idle state of the demo **
    case DEMO_STATE_IDLE:                   // Loops here until attach event
        break;
    
    // ** Running state of the demo **
    case DEMO_STATE_RUNNING:                // Loops while commands are in progress
        break;

    // ** Verify the device firmware **
    case DEMO_STATE_GET_DEV_VERSION:
        DataPacket.CMD = READ_VERSION;
        DataPacket.len = READ_VERSION_CMD_SIZE;
        if (SendDeviceCommand(DevAddr, &DataPacket))
            DemoState = DEMO_STATE_RUNNING;
        break;

    // ** Read Temperature Sensor Data **
    case DEMO_STATE_GET_TEMPERATURE:
        DataPacket.CMD = RD_TEMP;
        DataPacket.len = RD_TEMP_CMD_SIZE;
        if (SendDeviceCommand(DevAddr, &DataPacket))
            DemoState = DEMO_STATE_RUNNING;
        break;

    /**********************************************************************
     * LAB3 Step 4a - Read the Potentiometer Data                         *
     **********************************************************************
     **********************  I N T E R M E D I A T E  *********************
     **********************************************************************
     *  1.  Remove the temporary implementation of the DEMO_STATE_GET_POT
     *      state transition below.
     *  2.  "Un-comment" the implementation that sends the RD_POT command
     *      to start the sequence to read the potentiometer value.
     */
        
    // ** Read POT Sensor Data **
    case DEMO_STATE_GET_POT:
        DataPacket.CMD = RD_POT;
        DataPacket.len = RD_POT_CMD_SIZE;
        if (SendDeviceCommand(DevAddr, &DataPacket))
            DemoState = DEMO_STATE_RUNNING;
        break;

    /*
     * Temporary Code
     *

    case DEMO_STATE_GET_POT:
        DemoState = DEMO_STATE_GET_TEMPERATURE;
        break;

     *
     **********************************************************************
     * END LAB3 Step 4a                                                   *
     **********************************************************************/


    /**********************************************************************
     * LAB3 Step 5a - Add New Feature                                     *
     **********************************************************************
     **************************  A D V A N C E D  *************************
     **********************************************************************
     * Add a new state to check when SW3 and SW6 have been pressed.  Each 
     * time SW3 or SW6 is pressed toggle the data item for that switch and 
     * send the appropriate command to the device to update the state of 
     * its LEDs.  Be sure to add code to the appropriate event-handlers to 
     * read the response when the command has been sent and to update the
     * host's own LEDs to track the device's LEDs after the response has 
     * been read.
     */

    case DEMO_STATE_SET_LED:
        if (UpdateDeviceLEDs())
            DemoState = DEMO_STATE_RUNNING;
        else
            DemoState = DEMO_STATE_GET_TEMPERATURE;
        break;

    /*
     **********************************************************************
     * END LAB3 Step 5a                                                   *
     **********************************************************************/


    // ** Error state:  Hold here until detached **
    case DEMO_STATE_ERROR:
        break;

    default:
        // If we should somehow happen to hit an invalid state, re-initialize
        DemoState = DEMO_INITIALIZE;
        break;
    }

} // ManageDemoState

/*********************
 * USB Event Support *
 *********************/

/*************************************************************************
 Function:        DemoAttachEventHandler
 
 Preconditions:   A EVENT_GENERIC_ATTACH event has occured. 
                  
 Input:           pDevID      Pointer to the GENERIC_DEVICE_ID data
 
                  size        Should be sizeof(GENERIC_DEVICE_ID)
 
 Output:          DevAddr (global)
                  Updates device address when an attach occurs.
 
                  DemoState (global)
                  Updates the demo state as appropriate.
 
 Returns:         TRUE if the event was handled, FALSE if not
 
 Side Effects:    App State has been changed if data was valid.
 
 Overview:        This routine validates the attach-event data and
                  changes the state of the application appropriately.
 *************************************************************************/

BOOL DemoAttachEventHandler ( GENERIC_DEVICE_ID *pDevID, DWORD size )
{
    // If data is valid, store the device address and change state
    if ( pDevID != NULL  &&  size == sizeof(GENERIC_DEVICE_ID) )
    {
        if ( pDevID->vid == 0x04D8  &&  pDevID->pid == 0x000C )
        {
            DevAddr   = pDevID->deviceAddress;
            DemoState = DEMO_STATE_GET_DEV_VERSION;
            return TRUE;
        }
    }

    // Otherwise, we have an error.
    LCDWriteLine( 2, "Dev Attach Error" );
    DemoState = DEMO_STATE_ERROR;

    return FALSE;

} // DemoAttachEventHandler


/*************************************************************************
 Function:        TxDoneEventHandler

 Preconditions:   A EVENT_GENERIC_TX_DONE event has occured. 

 Input:           packet      Pointer to the packet sent
 
                  size        Size of the packet

 Output:          DemoState (global)
                  Updates the demo state as appropriate.

 Returns:         TRUE if the event was handled, FALSE if not

 Side Effects:    App State may been changed or response read started

 Overview:        This handles reading the response after a command 
                  packet has been sent to the device.
*************************************************************************/

BOOL TxDoneEventHandler ( DATA_PACKET *TxData, DWORD size )
{
    // Double check data packet pointer
    if ( TxData == &DataPacket )
    {
        // Read the response from each command
        switch (DataPacket.CMD)
        {
        case READ_VERSION:
            if (!ReadDeviceResponse(DevAddr, &DataPacket, READ_VERSION_RESP_SIZE))
            {
                LCDWriteLine( 2, "Err Reading Ver " );
                DemoState = DEMO_STATE_ERROR;
            }
            break;

        case RD_TEMP:
            if (!ReadDeviceResponse(DevAddr, &DataPacket, RD_TEMP_RESP_SIZE))
            {
                LCDWriteLine( 2, "Err Reading Temp" );
                DemoState = DEMO_STATE_ERROR;
            }
            break;


        /**********************************************************************
         * LAB3 Step 4e - Read Response to RD_POT Command                     *
         **********************************************************************
         **********************  I N T E R M E D I A T E  *********************
         **********************************************************************
         *  1.  "Un-comment" the lines below to read the response to the RD_POT
         *      command.
         */
            
        case RD_POT:
            if (!ReadDeviceResponse(DevAddr, &DataPacket, RD_POT_RESP_SIZE))
            {
                LCDWriteLine( 2, "Err Reading Pot " );
                DemoState = DEMO_STATE_ERROR;
            }
            break;
        
        /*
         **********************************************************************
         * END LAB3 Step 4e                                                   *
         **********************************************************************/

        case UPDATE_LED:
            if (!ReadDeviceResponse(DevAddr, &DataPacket, UPDATE_LED_RESP_SIZE))
            {
                LCDWriteLine( 2, "Err Reading LEDs" );
                DemoState = DEMO_STATE_ERROR;
            }
            break;

        default:
            // Unknown command sent (shouldn't happen)
            LCDWriteLine( 2, "Bad Command Sent" );
            DemoState = DEMO_STATE_ERROR;
        }

    }
    else
    {
        // Invalid packet pointer
        LCDWriteLine( 2, "Packet Send Err " );
        DemoState = DEMO_STATE_ERROR;
    }

    // Return FALSE if we're in an error state.
    if (DemoState == DEMO_STATE_ERROR)
    {
        return FALSE;
    }

    // Invalidate command so we can verify response
    DataPacket.CMD = CMD_INVALID;

    return TRUE;

} // TxDoneEventHandler


/*************************************************************************
 Function:        RxDoneEventHandler

 Preconditions:   A EVENT_GENERIC_RX_DONE event has occured. 

 Input:           packet      Pointer to the packet received
 
                  size        Size of the packet

 Output:          DemoState (global)
                  Updates the demo state as appropriate.

 Returns:         TRUE if the event was handled, FALSE if not

 Side Effects:    App State has been changed and data has been displayed
                  if it was valid.

 Overview:        This routine handles data received from the device,
                  displaying it as appropriate.
*************************************************************************/

static inline BOOL RxDoneEventHandler ( DATA_PACKET *RxData, DWORD size )
{
    // Double check data packet size & pointer.
    if ( RxData == &DataPacket && size == ResponseSize )
    {
        // Check the demo state, handle the response, & update the demo state.
        switch (DataPacket.CMD)
        {
        case READ_VERSION:
            // Validate and display the version number on the LCD
            if (ValidateAndDisplayDeviceFwVersion(RxData))
                DemoState = DEMO_STATE_GET_TEMPERATURE;
            break;

        case RD_TEMP:
            // Display the temprature data on the LCD
            if (DisplayTemperature(RxData))
                DemoState = DEMO_STATE_GET_POT;
            break;


        /**********************************************************************
         * LAB3 Step 4g - Display POT Data                                    *
         **********************************************************************
         **********************  I N T E R M E D I A T E  *********************
         **********************************************************************
         *  1.  "Un-comment" the lines below to display the response to the 
         *      RD_POT command.
         */
            
        case RD_POT:
            // Display the POT data on the LCD
            if (DisplayPot(RxData))
                DemoState = DEMO_STATE_SET_LED;
            break;
        
        /*
         **********************************************************************
         * END LAB3 Step 4g                                                   *
         **********************************************************************/

        case UPDATE_LED:
            if (UpdateHostLEDs(&DataPacket))
                DemoState = DEMO_STATE_GET_TEMPERATURE;
            break;

        default:
            // Unknown response packet error.
            LCDWriteLine( 2, "Unknown Resp Err" );
            DemoState = DEMO_STATE_ERROR;
            break;
        }
    }
    else
    {
        // Invalid repsonse packet
        LCDWriteLine( 2, "Resp Packet Err " );
        DemoState = DEMO_STATE_ERROR;
    }

    // Return FALSE if we're in an error state.
    if (DemoState == DEMO_STATE_ERROR)
    {
        return FALSE;
    }
        
    return TRUE;

} // RxDoneEventHandler


/*************************************************************************
 Function:        usb_custom_host_demo_EventHandler
 
 Preconditions:   The USB must be initialized. 
                  
 Input:           event       Identifies the bus event that occured
 
                  data        Pointer to event-specific data
 
                  size        Size of the event-specific data
 
 Output:          DevAddr (global)
                  Updates device address when an attach or detach occurs.
 
                  DemoState (global)
                  Updates the demo state as appropriate when events occur.
 
 Returns:         TRUE if the event was handled, FALSE if not
 
 Side Effects:    Event-specific actions have been taken.
 
 Overview:        This routine is called by the Host layer or client
                  driver to notify the application of events that occur.
                  If the event is recognized, it is handled and the 
                  routine returns TRUE.  Otherwise, it is ignored (or 
                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL usb_custom_host_demo_EventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    BYTE cnt;

    // Handle specific events.
    switch (event)
    {

    /**********************************************************************
     * LAB3 Step 3c - Recognize Device Attach Event                       *
     **********************************************************************
     **************************  B E G I N N E R  *************************
     **********************************************************************
     *  1.  "Un-comment" the lines below to recognize when the device has
     *      been attached.
     */

    case EVENT_GENERIC_ATTACH:
        return DemoAttachEventHandler((GENERIC_DEVICE_ID *)data, size);

    /*
     **********************************************************************
     * END LAB3 Step 3c                                                   *
     **********************************************************************/


    case EVENT_GENERIC_DETACH:
        DevAddr   = 0;
        DemoState = DEMO_INITIALIZE;    // Re-initialize the demo
        return TRUE;

    case EVENT_GENERIC_TX_DONE:
        return TxDoneEventHandler( data, size );

    case EVENT_GENERIC_RX_DONE:
        return RxDoneEventHandler( data, size );

    case EVENT_VBUS_REQUEST_POWER:
        // The data pointer points to a byte that represents the amount of power
        // requested in mA, divided by two.  If the device wants too much power,
        // we reject it.
        if (((USB_VBUS_POWER_EVENT_DATA*)data)->current <= (MAX_ALLOWED_CURRENT / 2))
        {
            return TRUE;
        }
        LCDWriteLine( 2, "Over Current Err" );
        DemoState = DEMO_STATE_ERROR;
        break;

    case EVENT_VBUS_RELEASE_POWER:
        // We cannot turn off VBus power through software with the Explorer 16
        // However, this will identify a detach of an un-supported device.
        DevAddr   = 0;
        DemoState = DEMO_INITIALIZE;    // Re-initialize the demo
        return TRUE;

    case EVENT_UNSUPPORTED_DEVICE:
        LCDWriteLine( 2, "No Dev Support! " );
        DemoState = DEMO_STATE_ERROR;
        break;

    case EVENT_HUB_ATTACH:
        LCDWriteLine( 2, "No Hub Support! " );
        DemoState = DEMO_STATE_ERROR;
        break;

    // Handle Errors
    case EVENT_BUS_ERROR:
    case EVENT_CANNOT_ENUMERATE:
    case EVENT_OUT_OF_MEMORY:
    case EVENT_UNSPECIFIED_ERROR:
        LCDWriteLine( 2, "Bus Error!      " );
        DemoState = DEMO_STATE_ERROR;
        return TRUE;

    default:
        break;
    }

    // Let the host layer know we ignore all other events.
    return FALSE;

} // usb_custom_host_demo_EventHandler



//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    
    // Initialize the USB
    USBInitialize(0);
    
    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

	return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

