/*******************************************************************************

    USB Host Generic Class Header File

This is the header file for the USB Host Generic Class driver.  It is required
for use with USBHostGeneric.c.

*******************************************************************************/
/******************************************************************************

* FileName:        USBHostGeneric.h
* Dependencies:    None
* Processor:       PIC24/dsPIC30/dsPIC33/PIC32MX
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the Company) for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

Author          Date    Comments
--------------------------------------------------------------------------------
BC/KO       25-Dec-2007 First release

*******************************************************************************/
#ifndef __USB_HOST_GENERIC_H__
#define __USB_HOST_GENERIC_H__


// *****************************************************************************
// *****************************************************************************
// Section: Constants
// *****************************************************************************
// *****************************************************************************

// Generic client requires transfer event support.
#ifndef USB_ENABLE_TRANSFER_EVENT
    #error "Generic client driver requires transfer event (USB_ENABLE_TRANSFER_EVENT) support."
#endif

// Default Generic Driver Enpoint
#ifndef USB_GENERIC_EP
    #define USB_GENERIC_EP       1
#endif

// *****************************************************************************
// *****************************************************************************
// Section: USB Generic Client Events
// *****************************************************************************
// *****************************************************************************

// Allows the application to add a non-zero offset to the generic events
// to resolve conflicts in event number.
#ifndef EVENT_GENERIC_OFFSET
#define EVENT_GENERIC_OFFSET 0
#endif


/**********************************************************************
 * LAB3 Step 1f - Define Driver Specific Events                       *
 **********************************************************************
 **************************  B E G I N N E R  *************************
 **********************************************************************
 *  1.  "Un-comment" the code below to define the Generic client's 
 *      driver-specific events.
 */
 
        // Indicates that a Generic device has been attached.
        // data: Points to a GENERIC_DEVICE_ID structure
        // size: sizeof(GENERIC_DEVICE_ID)
#define EVENT_GENERIC_ATTACH  (EVENT_GENERIC_BASE+EVENT_GENERIC_OFFSET+0)

        // Indicates that the specified device has been detached from the USB.
        // data: Points to a BYTE that contains the device address.
        // size: sizeof(BYTE)
#define EVENT_GENERIC_DETACH  (EVENT_GENERIC_BASE+EVENT_GENERIC_OFFSET+1)

        // Indicates that a previous write request has completed.  These events
        // are enabled if USB Embedded Host transfer events are enabled
        // (USB_ENABLE_TRANSFER_EVENT is defined).
        // data: Pointer to the buffer that completed.
        // size: Actual number of bytes written to the device.
#define EVENT_GENERIC_TX_DONE (EVENT_GENERIC_BASE+EVENT_GENERIC_OFFSET+2)

        // Indicates that a previous read request has completed.  These events
        // are enabled if USB Embedded Host transfer events are enabled
        // (USB_ENABLE_TRANSFER_EVENT is defined).
        // data: Pointer to the buffer that completed.
        // size: Actual number of bytes read from the device.
#define EVENT_GENERIC_RX_DONE (EVENT_GENERIC_BASE+EVENT_GENERIC_OFFSET+3)

/*
 **********************************************************************
 * END LAB3 Step 1f                                                   *
 **********************************************************************/


// *****************************************************************************
// *****************************************************************************
// Section: USB Data Structures
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Generic Device ID Information

This structure contains identification information about an attached device.
*/
typedef struct _GENERIC_DEVICE_ID
{
    WORD        vid;                    // Vendor ID of the device
    WORD        pid;                    // Product ID of the device
    BYTE        deviceAddress;          // Address of the device on the USB
} GENERIC_DEVICE_ID;


// *****************************************************************************
/* Generic Device Information

This structure contains information about an attached device, including
status flags and device identification.
*/
typedef struct _GENERIC_DEVICE
{
    GENERIC_DEVICE_ID   ID;             // Identification information about the device
    DWORD               rxLength;       // Number of bytes received in the last IN transfer

        BYTE            rxErrorCode;    // Error code of last IN transfer
        BYTE            txErrorCode;    // Error code of last OUT transfer
    union
    {
        BYTE val;                       // BYTE representation of device status flags
        struct
        {
            BYTE initialized    : 1;    // Driver has been initialized
            BYTE txBusy         : 1;    // Driver busy transmitting data
            BYTE rxBusy         : 1;    // Driver busy receiving data
        };
    } flags;                            // Generic client driver status flags

} GENERIC_DEVICE;


// *****************************************************************************
// *****************************************************************************
// Section: USB Data Structures
// *****************************************************************************
// *****************************************************************************

#define API_VALID(a) ( (((a)==gc_DevData.ID.deviceAddress) && gc_DevData.flags.initialized == 1) ? TRUE : FALSE )


// *****************************************************************************
// *****************************************************************************
// Section: Global Variables
// *****************************************************************************
// *****************************************************************************

extern GENERIC_DEVICE   gc_DevData; // Information about the attached device.

// *****************************************************************************
// *****************************************************************************
// Section: Function Prototypes and Macro Functions
// *****************************************************************************
// *****************************************************************************


/******************************************************************************
 * Function:        void USBHostGenericWrite( BYTE deviceAddress,
 *                           BYTE *buffer, DWORD length )
 *
 * PreCondition:    The device must be connected and enumerated.
 *
 * Input:           deviceAddress   - USB Address of the device.
 *                  buffer          - Pointer to the data buffer
 *                  length          - Number of bytes to be transferred
 *
 * Output:          None - None
 *
 * Return Values:   USB_SUCCESS         - The Write was started successfully
 *                  (USB error code)    - The Write was not started.  See
 *                                          USBHostWrite() for a list of errors.
 *
 * Overview:        Use this routine to transmit data from memory to the device.
 *
 *                  Usage:
 *
 *                  if(!USBHostGenericTxIsBusy(deviceAddress))
 *                      USBHostGenericWrite(deviceAddress, &Buffer, sizeof(Buffer));
 *
 * Note:            None
 *****************************************************************************/

BYTE USBHostGenericWrite( BYTE deviceAddress, void *buffer, DWORD length);


/*******************************************************************************
Function:        void USBHostGenericRead( BYTE deviceAddress, BYTE *buffer,
                                         DWORD length )

PreCondition:    The device must be connected and enumerated.

Input:           deviceAddress  - USB Address of the device.
                 buffer         - Pointer to the data buffer
                 length         - Number of bytes to be transferred

Output:          None - None

Return Values:   USB_SUCCESS         - The Read was started successfully
                 (USB error code)    - The Read was not started.  See
                                       USBHostRead() for a list of errors.

Overview:        Use this routine to receive from the device into memory.

Example:
                 if(!USBHostGenericRxIsBusy(deviceAddress))
                     USBHostGenericRead(deviceAddress, &buffer, sizeof(buffer));

Note:            None
*******************************************************************************/

BYTE USBHostGenericRead( BYTE deviceAddress, void *buffer, DWORD length);


// *****************************************************************************
// *****************************************************************************
// Section: Host Stack Interface Functions
// *****************************************************************************
// *****************************************************************************

/******************************************************************************
Function:       BOOL USBHostGenericInit ( BYTE address, DWORD flags )

Precondition:   The device has been configured.

Input:          BYTE address    - Device's address on the bus
                DWORD flags     - Initialization flags

Output:         None - None

Return Values:  TRUE    - Initialization was successful
                FALSE   - Initialization failed

Overview:       This routine is a call out from the USB Host layer to
                the USB generic client driver.  It is called when a
                Microchip "generic" device has been connected to the
                host.  Its purpose is to initialize and activate the
                USB Generic client driver.

Note:           There may be multiple client drivers.  If so, the
                USB Host layer will call the initialize routine
                for each one of the clients that are required.
*******************************************************************************/

BOOL USBHostGenericInit ( BYTE address, DWORD flags );


/******************************************************************************
Function:       BOOL USBHostGenericEventHandler ( BYTE address, USB_EVENT event,
                            void *data, DWORD size )

Preconditions:  None

Input:          BYTE address    - Address of device with the event
                USB_EVENT event - The bus event that occured
                void *data      - Pointer to event-specific data
                DWORD size      - Size of the event-specific data

Output:         None - None

Return Values:  TRUE    - The event was handled
                FALSE   - The event was not handled

Overview:       This routine is called by the Host layer to notify
                the general client of events that occur.  If the event
                is recognized, it is handled and the routine returns
                TRUE.  Otherwise, it is ignored and the routine
                returns FALSE.
*******************************************************************************/

BOOL USBHostGenericEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size );


#endif
/*************************************************************************
 * EOF
 */

