/******************************************************************************

  USB Host Generic Client Driver

This is the Generic client driver file for a USB Host device.  This file
should be used in a project with usb_host.c to provided the USB hardware
interface.

To interface with usb_host.c, the routine USBHostGenericInit() should be
specified as the Initialize() function, and USBHostGenericEventHandler() should
be specified as the EventHandler() function in the usbClientDrvTable[] array.

Since the generic class is performed with interrupt transfers,
USB_SUPPORT_INTERRUPT_TRANSFERS must be defined.

*******************************************************************************/
/******************************************************************************

* FileName:        usb_client_generic.c
* Dependencies:    None
* Processor:       PIC24/dsPIC30/dsPIC33/PIC32MX
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the Company) for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

Author          Date    Comments
--------------------------------------------------------------------------------
BC/KO       25-Dec-2007 First release
BC          07-May-2008 Modified for MASTERs class, event-driven implementation.

*******************************************************************************/

#include "GenericTypeDefs.h"
#include "USB\usb.h"
#include "usb_client_generic.h"


// *****************************************************************************
// *****************************************************************************
// Section: Global Variables
// *****************************************************************************
// *****************************************************************************

GENERIC_DEVICE  gc_DevData;


// *****************************************************************************
// *****************************************************************************
// Section: Host Stack Client Driver Interface Functions
// *****************************************************************************
// *****************************************************************************

/*************************************************************************
 * Function:        USBHostGenericInit
 *
 * Precondition:    The device has been configured.
 *
 * Input:           address     Device's address on the bus
 *                  flags       Initialization flags
 *                  pCfgDesc    Pointer to the device's configuration
 *                              descriptor.
 *
 * Output:          none
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    The USB General client driver has been initialized.
 *
 * Overview:        This routine is a call out from the USB Host layer to
 *                  the USB generic client driver.  It is called when a
 *                  Microchip "generic" device has been connected to the
 *                  host.  Its purpose is to initialize and activate the
 *                  USB Generic client driver.
 *
 * Note:            There may be multiple client drivers.  If so, the
 *                  USB Host layer will call the initialize routine
 *                  for each one of the clients that are required.
 *************************************************************************/

BOOL USBHostGenericInit ( BYTE address, DWORD flags )
{
    BYTE *pDesc;

    // Initialize state
    gc_DevData.rxLength  = 0;
    gc_DevData.flags.val = 0;

    // Save device the address, VID, & PID
    gc_DevData.ID.deviceAddress = address;
    pDesc  = USBHostGetDeviceDescriptor(address);
    pDesc += 8;
    gc_DevData.ID.vid  =  (WORD)*pDesc;       pDesc++;
    gc_DevData.ID.vid |= ((WORD)*pDesc) << 8; pDesc++;
    gc_DevData.ID.pid  =  (WORD)*pDesc;       pDesc++;
    gc_DevData.ID.pid |= ((WORD)*pDesc) << 8; pDesc++;

    // Generic Client Driver Init Complete.
    gc_DevData.flags.initialized = 1;


    /**********************************************************************
     * LAB3 Step 3a - Call the Application Event Handler                  *
     **********************************************************************
     **************************  B E G I N N E R  *************************
     **********************************************************************
     *  1.  "Un-comment" the lines below to call the application's event-
     *      handling routine when the device has been attached.
     *

    // Notify that application that we've been attached to a device.
    USB_HOST_APP_EVENT_HANDLER(address, EVENT_GENERIC_ATTACH, &(gc_DevData.ID), sizeof(GENERIC_DEVICE_ID) );

     *
     **********************************************************************
     * END LAB3 Step 3a                                                   *
     **********************************************************************/


    return TRUE;

} // USBHostGenericInit


/*************************************************************************
 * Function:        USBHostGenericEventHandler
 *
 * Preconditions:   TBD.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          none
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer to notify
 *                  the general client of events that occur.  If the event
 *                  is recognized, it is handled and the routine returns
 *                  TRUE.  Otherwise, it is ignored and the routine
 *                  returns FALSE.
 *************************************************************************/

BOOL USBHostGenericEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    // Make sure it was for our device
    if ( address != gc_DevData.ID.deviceAddress)
    {
        return FALSE;
    }

    // Handle specific events.
    switch (event)
    {
    case EVENT_DETACH:
        // Notify that application that the device has been detached.
        USB_HOST_APP_EVENT_HANDLER(gc_DevData.ID.deviceAddress, EVENT_GENERIC_DETACH, &gc_DevData.ID.deviceAddress, sizeof(BYTE) );
        gc_DevData.flags.val        = 0;
        gc_DevData.ID.deviceAddress = 0;
        return TRUE;

    case EVENT_TRANSFER:
        // If the parameters are correct
        if ( (data != NULL) && (size == sizeof(HOST_TRANSFER_DATA)) )
        {
            HOST_TRANSFER_DATA *pXfer = (HOST_TRANSFER_DATA *)data;


            /**********************************************************************
             * LAB3 Step 3j - Call the Application Event Handler                  *
             **********************************************************************
             **************************  B E G I N N E R  *************************
             **********************************************************************
             *  1.  "Un-comment" the lines below to call the application's event-
             *      handling routine when a transfer has finished.
             *

            // Notifiy the application of the completion of an Rx or Tx transfer.
            if ( pXfer->bEndpointAddress == (USB_IN_EP|USB_GENERIC_EP) )
            {
                gc_DevData.flags.rxBusy = 0;
                gc_DevData.rxLength     = pXfer->dataCount;
                USB_HOST_APP_EVENT_HANDLER(gc_DevData.ID.deviceAddress, EVENT_GENERIC_RX_DONE,
                                           pXfer->pUserData, pXfer->dataCount );
            }
            else if ( pXfer->bEndpointAddress == (USB_OUT_EP|USB_GENERIC_EP) )
            {
                gc_DevData.flags.txBusy = 0;
                USB_HOST_APP_EVENT_HANDLER(gc_DevData.ID.deviceAddress, EVENT_GENERIC_TX_DONE,
                                           pXfer->pUserData, pXfer->dataCount );
            }
            else
            {
                return FALSE;
            }

             *
             **********************************************************************
             * END LAB3 Step 3j                                                   *
             **********************************************************************/


            return TRUE;
        }
        break;

    default:
        break;
    }

    return FALSE;
} // USBHostGenericEventHandler


// *****************************************************************************
// *****************************************************************************
// Section: Application Program Interface (API) Functions
// *****************************************************************************
// *****************************************************************************

/******************************************************************************
 * Function:        void USBHostGenericWrite( BYTE deviceAddress,
 *                              BYTE *buffer, DWORD length )
 *
 * PreCondition:    The device must be connected and enumerated.
 *
 * Input:           deviceAddress   - USB Address of the device.
 *                  buffer          - Pointer to the data buffer
 *                  length          - Number of bytes to be transferred
 *
 * Output:          None
 *
 * Returns:         USB_SUCCESS if successful or a USB error code if not.
 *
 * Side Effects:    If no Tx transfer was started, a new one has been.
 *
 * Overview:        Use this routine to transmit data from memory to the device.
 *
 *                  Usage:
 *
 *                  if(!USBHostGenericTxIsBusy(deviceAddress))
 *                      USBHostGenericWrite(deviceAddress, &Buffer, sizeof(Buffer));
 *
 * Note:            None
 *****************************************************************************/

BYTE USBHostGenericWrite( BYTE deviceAddress, void *buffer, DWORD length )
{
    BYTE RetVal;

    // Validate the call
    if (!API_VALID(deviceAddress)) return USB_INVALID_STATE;
    if (gc_DevData.flags.txBusy)   return USB_BUSY;

    // Set the busy flag and start a new OUT transfer.
    gc_DevData.flags.txBusy = 1;
    RetVal = USBHostWrite( deviceAddress, USB_OUT_EP|USB_GENERIC_EP, (BYTE *)buffer, length );
    if (RetVal != USB_SUCCESS)
    {
        gc_DevData.flags.txBusy = 0;    // Clear flag to allow re-try
    }

    return RetVal;

} // USBHostGenericWrite


/******************************************************************************
 * Function:        void USBHostGenericRead( BYTE deviceAddress,
 *                              BYTE   *buffer, DWORD length )
 *
 * PreCondition:    The device must be connected and enumerated.
 *
 * Input:           deviceAddress   - USB Address of the device.
 *                  buffer          - Pointer to the data buffer
 *                  length          - Number of bytes to be transferred
 *
 * Output:          None
 *
 * Returns:         USB_SUCCESS if successful or a USB error code if not.
 *
 * Side Effects:    If no Rx transfer was started, a new one has been.
 *
 * Overview:        Use this routine to receive from the device into memory.
 *
 *                  Usage:
 *
 *                  if(!USBHostGenericRxIsBusy(deviceAddress))
 *                      USBHostGenericRead(deviceAddress, &Buffer, sizeof(Buffer));
 *
 * Note:            None
 *****************************************************************************/

BYTE USBHostGenericRead( BYTE deviceAddress, void *buffer, DWORD length )
{
    BYTE RetVal;

    // Validate the call
    if (!API_VALID(deviceAddress)) return USB_INVALID_STATE;
    if (gc_DevData.flags.rxBusy)   return USB_BUSY;

    // Set the busy flag, clear the count and start a new IN transfer.
    gc_DevData.flags.rxBusy = 1;
    gc_DevData.rxLength = 0;
    RetVal = USBHostRead( deviceAddress, USB_IN_EP|USB_GENERIC_EP, (BYTE *)buffer, length );
    if (RetVal != USB_SUCCESS)
    {
        gc_DevData.flags.rxBusy = 0;    // Clear flag to allow re-try
    }

    return RetVal;

} // USBHostGenericRead


/*************************************************************************
 * EOF usb_client_generic.c
 */


