/*
********************************************************************************
                                                                                
Software License Agreement                                                      
                                                                                
Copyright  2008 Microchip Technology Inc. and its licensors.  All         
rights reserved.                                                                
                                                                                
Microchip licenses to you the right to: (1) install Software on a single        
computer and use the Software with Microchip 16-bit microcontrollers and        
16-bit digital signal controllers ("Microchip Product"); and (2) at your        
own discretion and risk, use, modify, copy and distribute the device            
driver files of the Software that are provided to you in Source Code;           
provided that such Device Drivers are only used with Microchip Products         
and that no open source or free software is incorporated into the Device        
Drivers without Microchip's prior written consent in each instance.             
                                                                                
You should refer to the license agreement accompanying this Software for        
additional information regarding your rights and obligations.                   
                                                                                
SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY         
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY              
WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A          
PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE             
LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY,               
CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY           
DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY         
INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR         
LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY,                 
SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY         
DEFENSE THEREOF), OR OTHER SIMILAR COSTS.                                       
                                                                                
********************************************************************************
*/

#include "GenericTypedefs.h"
#include "HardwareProfile.h"
#include "boot.h"
#include <plib.h>
#include "MDD File System\FSIO.h"
#include "USB\usb.h"


//*****************************************************************************
//*****************************************************************************
// Boot Loader Media Interface Definitions
//*****************************************************************************
//*****************************************************************************

// Check for definitions of required build parameters

// Maximum number of retries to mount the boot medium
#ifndef MAX_NUM_MOUNT_RETRIES
    #define MAX_NUM_MOUNT_RETRIES       10
#endif

// Defines the size of the buffer used to read the boot image file.
#ifndef BL_READ_BUFFER_SIZE
    #define BL_READ_BUFFER_SIZE        512
#endif


//******************************************************************************
//******************************************************************************
// Global Data
//******************************************************************************
//******************************************************************************

// Flag indicating media attached
BOOL            MediaPresent;

// Flag indicating file system volume valid
BOOL            VolumeValid;

// Buffer for reading image file data
BYTE            ReadBuffer[BL_READ_BUFFER_SIZE];

// Stores data and base address of data for use by the programming routine
FLASH_BLOCK     FlashBlock;


//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/****************************************************************************
  Function:
    BOOL USB_ThumbDriveEventHandler( BYTE address, USB_EVENT event,
                void *data, DWORD size )

  Summary:
    This is the boot loader event handler.  It is called when the USB stack 
    has an event that needs to be handled by the boot loader rather than by
    the USB client driver.

  Precondition:
    The USB stack must have been initialized.

  Parameters:
    address - Address of device where event occurred
    event   - Identifies the event that occured
    data    - Pointer to event-specific data
    size    - Size of the event-specific data

  Return Values:
    TRUE    - The event was handled
    FALSE   - The event was not handled

  Remarks:
    None
  ***************************************************************************/

BOOL USB_ThumbDriveEventHandler( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    switch( event )
    {        
    case EVENT_VBUS_REQUEST_POWER:
        // The data pointer points to a byte that represents the amount of power
        // requested in mA, divided by two.  If the device wants too much power,
        // we reject it.
        if (((USB_VBUS_POWER_EVENT_DATA*)data)->current <= (MAX_ALLOWED_CURRENT / 2))
        {
            return TRUE;
        }
        else
        {
            MediaPresent = FALSE;
            BLIO_ReportBootStatus( BL_USB_OC_ERR, "BL: USB Error - device requires too much current\r\n" );
        }
        break;

    case EVENT_VBUS_RELEASE_POWER:
        // The Explorer 16 cannot turn off Vbus through software.
        BLIO_ReportBootStatus( BL_MEDIUM_DETACH, "BL: USB - Boot medium detached\r\n" );
        return TRUE;

    case EVENT_HUB_ATTACH:
        BLIO_ReportBootStatus( BL_USB_HUB_ERR, "BL: USB Error - hubs are not supported\r\n" );
        return TRUE;

    case EVENT_UNSUPPORTED_DEVICE:
        BLIO_ReportBootStatus( BL_BAD_DEV_ERR, "BL: USB Error - device is not supported\r\n" );
        return TRUE;

    case EVENT_CANNOT_ENUMERATE:
        BLIO_ReportBootStatus( USB_ENUM_ERR,  "BL: USB Error - cannot enumerate device\r\n" );
        return TRUE;

    case EVENT_CLIENT_INIT_ERROR:
        BLIO_ReportBootStatus( USB_MSD_INIT_ERR, "BL: USB Error - msd client driver initialization error\r\n" );
        return TRUE;

    case EVENT_OUT_OF_MEMORY:
        BLIO_ReportBootStatus( OUT_OF_MEMORY_ERR, "BL: Error - out of heap memory\r\n" );
        return TRUE;

    case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
        BLIO_ReportBootStatus( USB_ERR, "BL: USB Error - unspecified\r\n" );
        return TRUE;

    }

    return FALSE;

} // USB_ThumbDriveEventHandler


//******************************************************************************
//******************************************************************************
// Boot Loader Media Interface Functions
//******************************************************************************
//******************************************************************************

/****************************************************************************
  Function:
    BOOL BLMedia_InitializeTransport ( void )

  Description:
    This routine is called to initalize the media transport layer used to 
    access the boot medium.

  Precondition:
    None

  Parameters:
    None

  Returns:
    TRUE    - If able to initialize the boot medium's transport layer
    FALSE   - If not able to initialize the boot medium's transport layer

  Remarks:
    This routine can be implemented to inialize any desired boot medium.
***************************************************************************/

// Implemented as a macro to call USBInitialize()


/****************************************************************************
  Function:
    void BLMedia_DeinitializeTransport ( void )

  Description:
    This routine disables the media transport layer that was initialized by
    BLMedia_InitializeTransport.


  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    This routine must disable and clear any interrupts that were enabled
    by BLMedia_InitializeTransport.  Any stray pending interrupts (once the
    application boots) may cause the system to trigger an un-handled (or 
    incorrectly handled) interrupt.
***************************************************************************/

// Implemented as a macro to call USBShutdown()


/****************************************************************************
  Function:
    BOOL BLMedia_MonitorMedia( void )

  Description:
    This function calls the background tasks necessary to support USB Host
    operation.  Upon initial insertion of the media, it initializes the file
    system support.

  Precondition:
    The USB transport must have been initialized.

  Parameters:
    None

  Returns:
    TRUE    - As long as the media interface is active
    FALSE   - If for any reason the media interface needs to be 
              re-initialized.

  Remarks:
    Calls USBTasks() to maintain the USB MSD and FAT FS stack.
  ***************************************************************************/

BOOL BLMedia_MonitorMedia ( void )
{
    BYTE            mediaPresentNow;
    BYTE            mountTries;

    USBTasks();

    mediaPresentNow = USBHostMSDSCSIMediaDetect();
    if (mediaPresentNow != MediaPresent)
    {
        if (mediaPresentNow)
        {
            mountTries = MAX_NUM_MOUNT_RETRIES;
            while(!FSInit() && mountTries--);
            if (!mountTries)
            {
                BLIO_ReportBootStatus(BL_FS_INIT_ERR, "BL: USB Error - Could not initialize file system\r\n" );
                MediaPresent = FALSE;
            }
            else
            {
                MediaPresent = TRUE;
            }
        }
        else
        {
            MediaPresent    = FALSE;
            VolumeValid     = FALSE;
        }
    }

    return TRUE;

} // MonitorMedia


/****************************************************************************
  Function:
    BOOL BLMedia_MediumAttached ( void )

  Description:
    This routine determines if the boot medium is currently attached to the
    system.

  Precondition:
    BLMedia_InitializeTransport must have been called and the medium muust
    be maintained as needed.

  Parameters:
    None

  Returns:
    TRUE    - If the boot medium is currently attached and accessible
    FALSE   - If the boot medium is not currently attached

  Remarks:
    None
***************************************************************************/

// Implemented as a macro to call USBHostMSDSCSIMediaDetect()


/****************************************************************************
  Function:
    BOOL BLMedia_LocateFile ( char *file_name )

  Description:
    Searches the thumb drive's root directory and identifies if 
    the given file is present.

  Precondition:
    The USB transport must have been initialized.

  Parameters:
    file_name - Pointer to a null-terminated character string giving the 
                name of the application's image file.

  Returns:
    TRUE      - If the application's image file named by the file_name string
                is able to be found on the boot medium.
    FALSE     - If the file cannot be found.

  Remarks:
    None
  ***************************************************************************/

BOOL BLMedia_LocateFile ( char *file_name )
{
    BOOL        fileFound;
    SearchRec   searchRecord;

    if (FindFirst( file_name, 
                   ATTR_DIRECTORY | ATTR_ARCHIVE | ATTR_READ_ONLY | ATTR_HIDDEN, 
                   &searchRecord ) == 0 )
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }

} // BLMedia_LocateFile


/****************************************************************************
  Function:
    BOOL BLMedia_LoadFile (  char *file_name )

  Description:
    Accesses the boot image file, reads it over the transport, and programs
    it to Flash.

  Precondition:
    The boot medium and file system must have been initialized and
    maintained as needed.

  Parameters:
    file_name - Pointer to a null-terminated character string giving the 
                name of the application's image file.

  Returns:
    TRUE      - If a valid application image was read from the thumb drive
                and programmed into Flash at the APPLICATION_ADDRESS.
    FALSE     - If the application image was not valid or if unable to 
                program a valid application to Flash for any reason.

  Remarks:
    This routine calls the loader layer to translate and program the boot
    image file.
    
    This routine can be modified to account for differences in how the medium
    and file format must be processed.
  ***************************************************************************/

BOOL BLMedia_LoadFile (  char *file_name )
{
    FSFILE         *fp;             // File pointer
    size_t          nBuffer;        // Number of bytes in the read buffer
    size_t          iBuffer;        // Index into the read buffer
    unsigned int    nRemaining;     // Number of bytes remaining to decode
    unsigned int    Result;         // Result code from "GetFlashBlock" operation

    // Attempt to open the file
    if ( (fp=FSfopen( file_name, "r" )) == NULL )
    {
        BLIO_ReportBootStatus(BL_FILE_ERR, "BL: USB Error - Unable to open file\r\n" );
        return FALSE;
    }
    else
    {
        Loader_Initialize(&FlashBlock);

        // Read the file and program it to Flash
        iBuffer     = 0;
        nRemaining  = 0;
        nBuffer     = BL_READ_BUFFER_SIZE;
        while ( (nBuffer=FSfread( &ReadBuffer[iBuffer], 1, nBuffer, fp )) != 0 )
        {
            // Decode the raw data and program as many Flash Blocks as we can.
            // Note:  Loader_GetFlashBlock updates nRemaining.
            iBuffer		= 0;
            nRemaining += nBuffer;
            nBuffer     = nRemaining;
            while ( (Result=Loader_GetFlashBlock(&FlashBlock, &nRemaining, &ReadBuffer[iBuffer])) == LOADER_BLOCK_READY )
            {
                // The Flash block is ready to program.
                if (!Loader_ProgramFlashBlock(&FlashBlock))
                {
                    // Error reported by the loader layer, close and fail.
                    FSfclose( fp );
                    return FALSE;
                }

                // Update the ReadBuffer index.
                iBuffer = nBuffer - nRemaining;
            }

            // Check "GetFlashBlock" result
            if (Result == LOADER_EOF)
            {
                // EOF record found!  Program the last block and close the file.
                Loader_ProgramFlashBlock(&FlashBlock);
                FSfclose( fp );
                return TRUE;
            }
            if (Result > LOADER_NEED_DATA)
            {
                // Error reported by the loader layer, close and fail.
                FSfclose( fp );
                return FALSE;
            }
            
            // Calculate the index and amount for the next read
            iBuffer = nBuffer - nRemaining;
            if (nRemaining > 0)
            {
                // Copy the remaining data to the top of the buffer and 
                // adjust the read size and starting position
                memcpy(ReadBuffer, &ReadBuffer[iBuffer], nRemaining);
                nBuffer = BL_READ_BUFFER_SIZE - nRemaining;
                iBuffer = nRemaining;
            }
            else
            {
                // Read a full buffer next time.
                nBuffer = BL_READ_BUFFER_SIZE;
                iBuffer = 0;
            }
        }

        FSfclose( fp );
    }

    return TRUE;

} // BLMedia_LoadFile


/*
*******************************************************************************
EOF
*******************************************************************************
*/

